/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef QUB_FITCURVES_H
#define QUB_FITCURVES_H


using namespace std;

#include "QUB_Freezer.h"
#include "QUB_NodeArray.h"


// signatures shared with Delphi

class QUB_FitCurve
{
public:
	virtual int            __cdecl  First() = 0;
	virtual int            __cdecl  Last() = 0;
	virtual int*           __cdecl  Data() = 0;
};

class QUB_FitCurveIter
{
public:
	virtual void           __cdecl  Free() = 0;
	virtual int            __cdecl  Done() = 0;
	virtual int            __cdecl  Next() = 0;
	virtual QUB_FitCurve*  __cdecl  Curve() = 0;
	virtual void           __cdecl  MarkChanged() = 0;
};

class QUB_FitCurves
{
public:
	virtual void              __cdecl  Free() = 0;
	virtual void              __cdecl  Clear() = 0;
	virtual QUB_FitCurveIter* __cdecl  Add(int first, int last) = 0;
	virtual QUB_FitCurveIter* __cdecl  Find(int first) = 0;
	virtual int               __cdecl  Empty() = 0;

	// automatically locked during method calls and while an iterator exists
	// (only locked against other threads)
	virtual void              __cdecl  lock() = 0;
	virtual void              __cdecl  unlock() = 0;
	virtual void              __cdecl  UseTempFile() = 0;
};

extern "C" MIL_API QUB_FitCurves *    __stdcall QUB_FitCurves_Create();


// implementing classes

class QUB_FitCurve_impl : public QUB_FitCurve
{
private:
	int first, last;
	QUB_Tree data;
	int storage;
	bool changed;

public:
	QUB_FitCurve_impl(int f, int l, QUB_Tree dataNode)
		: first( f ), last( l ), data( dataNode ), storage( QUB_FS_0 ), changed( false )
	{     data.setNumData(QTR_TYPE_INT, l - f + 1, 1, (void*) 0);    }

	virtual int            __cdecl  First()    { return first; }
	virtual int            __cdecl  Last()     { return last; }
	virtual int*           __cdecl  Data()     { return (int *) data.data(); }

	// for the freezer
	int size()                                 { return data.dataCount(); }
	void setupStorage(int lvl);
	void clearStorage(int lvl);
	int  getStorage()                          { return storage; }
	void MarkChanged()                         { changed = true; }
};

typedef QUB_Freezer<QUB_FitCurve_impl> QUB_FitFreezer;
typedef QUB_FreezerSeries<QUB_FitCurve_impl> QUB_FitFreezerSeries;
typedef map<int, int> QUB_FitMap; // curve.first -> index in fseries and nodearray

class QUB_FitCurves_impl;

class QUB_FitCurveIter_impl : public QUB_FitCurveIter
{
private:
	QUB_FitCurves_impl&    curves;
	QUB_FitFreezerSeries&  fseries;
	QUB_NodeArray&         nodes;
	QUB_FitMap&            locations;
	QUB_FitMap::iterator   locIter;
	QUB_FitCurve_impl*     curve;
	bool                   changed;

public:
	QUB_FitCurveIter_impl( QUB_FitCurves_impl& curvs, int first );
	virtual ~QUB_FitCurveIter_impl();

	virtual void           __cdecl  Free()        { delete this; }
	virtual int            __cdecl  Done()        { return (locIter == locations.end()); }
	virtual int            __cdecl  Next();
	virtual QUB_FitCurve*  __cdecl  Curve()       { return curve; }
	virtual void           __cdecl  MarkChanged() { changed = true; if ( curve ) curve->MarkChanged(); }
};

class QUB_FitCurves_impl : public QUB_FitCurves
{
private:
	QUB_FitFreezer         freezer;
	CRITICAL_SECTION       crit;

public:
	QUB_FitFreezerSeries   fseries;
	QUB_NodeArray          nodes;
	QUB_FitMap             locations;

	QUB_FitCurves_impl();
	virtual ~QUB_FitCurves_impl();

	virtual void              __cdecl  Free();
	virtual void              __cdecl  Clear();
	virtual QUB_FitCurveIter* __cdecl  Add(int first, int last);
	virtual QUB_FitCurveIter* __cdecl  Find(int first);
	virtual int               __cdecl  Empty();

	virtual void              __cdecl  lock();
	virtual void              __cdecl  unlock();
	virtual void              __cdecl  UseTempFile();
};


extern "C" MIL_API QUB_FitCurves*    __stdcall QUB_FitCurves_Create();



#endif
